#property copyright "ArteInVolo"
#property link      "https://vk.com/tradelearning"
#property version   "1.00"

#include <Object.mqh>
#include <MovingAverages.mqh>
#include <Arrays\ArrayObj.mqh>
#include <Controls\ComboBox.mqh>
#include "Statistics.mqh"

#property indicator_separate_window
#property indicator_buffers 10
#property indicator_plots   10

#property indicator_label1  "Currency"
#property indicator_type1   DRAW_LINE
#property indicator_color1  clrBlack
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1

#property indicator_label2  "Currency"
#property indicator_type2   DRAW_LINE
#property indicator_color2  clrBlue
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1

#property indicator_label3  "Currency"
#property indicator_type3   DRAW_LINE
#property indicator_color3  clrRed
#property indicator_style3  STYLE_SOLID
#property indicator_width3  1

#property indicator_label4  "Currency"
#property indicator_type4   DRAW_LINE
#property indicator_color4  clrSilver
#property indicator_style4  STYLE_SOLID
#property indicator_width4  1

#property indicator_label5  "Currency"
#property indicator_type5   DRAW_LINE
#property indicator_color5  clrLime
#property indicator_style5  STYLE_SOLID
#property indicator_width5  1

#property indicator_label6  "Currency"
#property indicator_type6   DRAW_LINE
#property indicator_color6  clrMagenta
#property indicator_style6  STYLE_SOLID
#property indicator_width6  1

#property indicator_label7  "Currency"
#property indicator_type7   DRAW_LINE
#property indicator_color7  clrOrange
#property indicator_style7  STYLE_SOLID
#property indicator_width7  1

#property indicator_label8  "Currency"
#property indicator_type8   DRAW_LINE
#property indicator_color8  clrDeepSkyBlue
#property indicator_style8  STYLE_SOLID
#property indicator_width8  1

#property indicator_label9  "Currency"
#property indicator_type9   DRAW_LINE
#property indicator_color9  clrAqua
#property indicator_style9  STYLE_SOLID
#property indicator_width9  1

#property indicator_label10  "Currency"
#property indicator_type10   DRAW_LINE
#property indicator_color10  clrDeepPink
#property indicator_style10  STYLE_SOLID
#property indicator_width10  1

// *** Input data *** //

input int InpPeriodFast = 5;
input int InpPeriodSlow = 200;
input int InpNumberOfBars = 500;
input color InpFontColor = Black;
input color InpSignalColor = Lime;
input string InpSymbols = "AUDUSD:EURAUD:EURGBP:EURJPY:EURUSD:GBPUSD:USDCAD:USDCHF:USDJPY";
input string InpActiveCurrency = "EURUSD";

// *** Indicator buffers *** //

double Currency1[], Currency2[], Currency3[], Currency4[], Currency5[], Currency6[], Currency7[], Currency8[], Currency9[];

// *** Global *** //

int FontSize = 13;
int YDistance = 5;
int XDistance = 235;
int ChartWindow = 0;
string FontName = "Trebuchet MS";

// *** Classes *** //

class CSymbol : public CObject
{
    public:
        string Name;
        double Close[];
        double SyncClose[];
        double AverageFast[];
        double AverageSlow[];
        datetime Time[];
        
        CSymbol(string name) { Name = name; };
};

// *** Objects *** //

CArrayObj * Currencies;
CComboBox * CurrencyBox;

// *** Functions *** //

void OnDeinit(const int reason) 
{
    delete(Currencies);
    delete(CurrencyBox);
}

int OnInit()
{
    SetIndexBuffer(0, Currency1,  INDICATOR_DATA);
    SetIndexBuffer(1, Currency2,  INDICATOR_DATA);
    SetIndexBuffer(2, Currency3,  INDICATOR_DATA);
    SetIndexBuffer(3, Currency4,  INDICATOR_DATA);
    SetIndexBuffer(4, Currency5,  INDICATOR_DATA);
    SetIndexBuffer(5, Currency6,  INDICATOR_DATA);
    SetIndexBuffer(6, Currency7,  INDICATOR_DATA);
    SetIndexBuffer(7, Currency8,  INDICATOR_DATA);
    SetIndexBuffer(8, Currency9,  INDICATOR_DATA);

    IndicatorSetString(INDICATOR_SHORTNAME, "CurrencyCorrelation");
    IndicatorSetInteger(INDICATOR_DIGITS, _Digits);
    
    ChartWindow = ChartWindowFind();

    PrepareVariables();
    PrepareControls();

    return(INIT_SUCCEEDED);
}

int OnCalculate(
    const int bars, 
    const int counted, 
    const datetime &time[], 
    const double &open[], 
    const double &high[], 
    const double &low[], 
    const double &close[], 
    const long &tickVolume[], 
    const long &volume[], 
    const int &spread[])
{
    Currency1[0] = 0;
    Currency2[0] = 0;
    Currency3[0] = 0;
    Currency4[0] = 0;
    Currency5[0] = 0;
    Currency6[0] = 0;
    Currency7[0] = 0;
    Currency8[0] = 0;
    Currency9[0] = 0;

    int calculated = bars;
    int start = MathMax(counted - 1, 1);
    int copied = Load(start, bars);

    if (copied > 0)
    {
        Synchronize(start, bars);
        ShowCorrelation(start, bars);
        ShowPanel(bars);
    }
    else
    {
        for (int i = start; i < bars; i++)
        {
            Currency1[i] = Currency1[i - 1];
            Currency2[i] = Currency2[i - 1];
            Currency3[i] = Currency3[i - 1];
            Currency4[i] = Currency4[i - 1];
            Currency5[i] = Currency5[i - 1];
            Currency6[i] = Currency6[i - 1];
            Currency7[i] = Currency7[i - 1];
            Currency8[i] = Currency8[i - 1];
            Currency9[i] = Currency9[i - 1];
        }
        
        calculated = start;
    }

    return(calculated);
}

int Load(const int start, const int bars)
{
    for (int i = 0; i < Currencies.Total(); i++)
    {
        CSymbol * s = Currencies.At(i);
        
        double closes[];
        datetime times[];
        
        int countTimes = CopyTime(s.Name, _Period, start - 1, bars, s.Time);
        int countCloses = CopyClose(s.Name, _Period, start - 1, bars, s.Close);

        if (countTimes <= 0 || countCloses <= 0)
        {
            return(0);
        }
    }

    return(1);
}

int Synchronize(const int start, const int bars)
{
    datetime currentTime = datetime(SeriesInfoInteger(_Symbol, 0, SERIES_LASTBAR_DATE));

    for (int i = 0; i < Currencies.Total(); i++)
    {
        CSymbol * s = Currencies.At(i);

        ArrayResize(s.SyncClose, bars);
        ArrayResize(s.AverageFast, bars);
        ArrayResize(s.AverageSlow, bars);

        int closeIndex = ArraySize(s.Close) - 1;

        s.SyncClose[bars - 1] = s.Close[closeIndex];

        for (int k = bars - 2; k > start - 1; k--)
        {
            if (GetBarByTime(s.Name, _Period, currentTime) > 0 && closeIndex > 0)
            {
                closeIndex--;
            }

            s.SyncClose[k] = s.Close[closeIndex];            
            currentTime -= PeriodSeconds();
        }

        ExponentialMAOnBuffer(bars, start + 1, 1, InpPeriodFast, s.SyncClose, s.AverageFast);
        SmoothedMAOnBuffer(bars, start + 1, 1, InpPeriodSlow, s.SyncClose, s.AverageSlow);
    }

    return(1);
}

int GetBarByTime(string symbol, ENUM_TIMEFRAMES period, datetime time)
{
    datetime last;
    SeriesInfoInteger(symbol, period, SERIES_LASTBAR_DATE, last);
    return(Bars(symbol, period, time, last) - 1);
}

int PrepareVariables()
{
    string symbols[];
    StringSplit(InpSymbols, ':', symbols);
    
    int symbolCount = ArraySize(symbols);
    Currencies = new CArrayObj();

    for (int i = 0; i < symbolCount; i++)
    {
        CSymbol * s = new CSymbol(symbols[i]);
        
        if (SymbolSelect(s.Name, true))
        {
            PlotIndexSetString(i, PLOT_LABEL, s.Name);
            Currencies.Add(s);
        }
    }
    
    return(0);
}

void PrepareControls()
{
    long chartWidth = ChartGetInteger(ChartID(), CHART_WIDTH_IN_PIXELS, ChartWindow);
    double chartShift = chartWidth * ChartGetDouble(ChartID(), CHART_SHIFT_SIZE, ChartWindow) / 100;
    int selectorX = int(chartWidth - chartShift + 115);
    int selectorY = 12;
    int selectorWidth = 100;
    int selectorHeight = 20;
    
    CurrencyBox = new CComboBox();
    CurrencyBox.Create(ChartID(), "Demo", ChartWindow, selectorX, selectorY, selectorX + selectorWidth, selectorY + selectorHeight);

    for (int i = 0; i < Currencies.Total(); i++)
    {
        CurrencyBox.ItemAdd(SymbolName(i,true));
    }
}
void ShowCorrelation(const int start, const int bars)
{
    CSymbol * s;

    for (int i = start; i < bars; i++)
    {
        s = Currencies.At(0);
        Currency1[i] = s ? s.AverageFast[i] / s.AverageSlow[i] : Currency1[i - 1];
        s = Currencies.At(1);
        Currency2[i] = s ? s.AverageFast[i] / s.AverageSlow[i] : Currency2[i - 1];
        s = Currencies.At(2);
        Currency3[i] = s ? s.AverageFast[i] / s.AverageSlow[i] : Currency3[i - 1];
        s = Currencies.At(3);
        Currency4[i] = s ? s.AverageFast[i] / s.AverageSlow[i] : Currency4[i - 1];
        s = Currencies.At(4);
        Currency5[i] = s ? s.AverageFast[i] / s.AverageSlow[i] : Currency5[i - 1];
        s = Currencies.At(5);
        Currency6[i] = s ? s.AverageFast[i] / s.AverageSlow[i] : Currency6[i - 1];
        s = Currencies.At(6);
        Currency7[i] = s ? s.AverageFast[i] / s.AverageSlow[i] : Currency7[i - 1];
        s = Currencies.At(7);
        Currency8[i] = s ? s.AverageFast[i] / s.AverageSlow[i] : Currency8[i - 1];
        s = Currencies.At(8);
        Currency9[i] = s ? s.AverageFast[i] / s.AverageSlow[i] : Currency9[i - 1];
    }
}

void ShowPanel(const int bars)
{
    CSymbol * activeSymbol = Currencies.At(0);

    int positionTop = 0;
    int positionCurrent = 0;
    int symbols = Currencies.Total();

    ShowLabel("Currency", "CURRENCY : ", YDistance + 10, FontSize - 5, InpFontColor);
    
    YDistance += 40;
 
    for (int i = 0; i < symbols; i++)
    {
        CSymbol * s = Currencies.At(i);
    
        if (StringCompare(s.Name, InpActiveCurrency) == 0)
        {
            activeSymbol = s;
        }
    }

    // Get math values for current symbol

    double activeSequence[];
    double activeSummary = 0;
    
    ArrayResize(activeSequence, ArraySize(activeSymbol.SyncClose));
    
    for (int i = 0; i < bars; i++)
    {
        activeSequence[i] = activeSymbol.AverageFast[i] / activeSymbol.AverageSlow[i];
        activeSummary += activeSequence[i];
    }

    double activeExpectation = activeSummary / bars;
    double activeVariance = Variance(activeSequence, activeExpectation);
    
    // Compare with other symbols
    
    for (int i = 0; i < symbols; i++)
    {
        CSymbol * s = Currencies.At(i);

        if (StringCompare(s.Name, InpActiveCurrency) == 0)
        {
            continue;
        }

        int colorIndex = PlotIndexGetInteger(i, PLOT_COLOR_INDEXES);
        color colorValue = color(PlotIndexGetInteger(i, PLOT_LINE_COLOR, 0));
        
        if (i >= symbols / 2)
        {
            positionTop = int(symbols / 2);
            XDistance = 115;
        }

        double sequence[];
        double summary = 0;
        
        ArrayResize(sequence, ArraySize(s.SyncClose));
        
        for (int k = 0; k < bars; k++)
        {
            sequence[k] = s.AverageFast[k] / s.AverageSlow[k];
            summary += sequence[k];
        }
        
        double expectation = summary / bars;
        double variance = Variance(sequence, expectation);
        double covariance = Cov(activeSequence, sequence, activeExpectation, expectation);
        double correlation = Corr(covariance, activeVariance, variance);
        
        string content = StringFormat("%s : %s", s.Name, DoubleToString(correlation, _Digits));
        
        ShowLabel("Currency" + s.Name, content, YDistance + 20 * (positionCurrent - positionTop), FontSize - 5, colorValue);
        positionCurrent++;
    }
}

void ShowLabel(const string name, const string value, const int positionTop, const int fontSize, const color fontColor)
{
    if (ObjectFind(0, name) < 0)
    {
        ObjectCreate(0, name, OBJ_LABEL, ChartWindow, 0, 0);
    }
    
    ObjectSetString(0, name, OBJPROP_TEXT, value);
    ObjectSetString(0, name, OBJPROP_FONT, FontName);
    ObjectSetInteger(0, name, OBJPROP_COLOR, fontColor);
    ObjectSetInteger(0, name, OBJPROP_FONTSIZE, fontSize);
    ObjectSetInteger(0, name, OBJPROP_ALIGN, ALIGN_RIGHT); 
    ObjectSetInteger(0, name, OBJPROP_XDISTANCE, XDistance);
    ObjectSetInteger(0, name, OBJPROP_YDISTANCE, positionTop);
    ObjectSetInteger(0, name, OBJPROP_CORNER, CORNER_RIGHT_UPPER); 
}